# -*- coding: utf-8 -*-

import collections
import re

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.forms.utils import ErrorList

from bbcode.forms import BBCodeModelForm
from club.models import AliasGere, ArticleBlog, Club
from club.views import TITRES
from edt.models import EdtClubs
from ldap_inp import LdapAlias
from lib.forms_inp import PhoneField
from portailuser.fields import PortailUserField
from portailuser.models import Ecole
from django.utils.safestring import mark_safe


ACTIVITES = TITRES.items()
TYPES = (
    ("club", "Club"),
    ("asso", "Association"),
)


def validation_nom_simple(valeur):
    suffixes = ["-" + ecole.ident for ecole in Ecole.objects.all()]

    for suffixe in suffixes:
        if valeur.endswith(suffixe):
            raise ValidationError("Le suffixe “{}” ne peut être utilisé pour " "un groupe.".format(suffixe))


class NouvClubForm(forms.Form):
    nom = forms.CharField(label="Nom du club")
    nom_simp = forms.CharField(
        label="Nom simplifié",
        validators=[RegexValidator(r"^[a-z0-9_-]+$"), validation_nom_simple],
        help_text="Caractères valides : a-z, 0-9, tiret, tiret bas.",
    )
    activite = forms.ChoiceField(label="Activité", choices=ACTIVITES)
    typeClub = forms.TypedChoiceField(label="Type", choices=TYPES)
    telephone = PhoneField(label="Nº de téléphone", required=False)
    numeroFax = PhoneField(label="Nº de fax", required=False)
    mailGroupe = forms.EmailField(label="Adresse email", required=False)
    local = forms.CharField(widget=forms.TextInput(attrs={"size": "5"}), required=False)
    president = PortailUserField(
        label="Président",
        help_text="Entrez un login INP-net, ou cliquez sur l’icône pour rechercher un membre.",
    )
    vicePresident = PortailUserField(label="Vice-Président", required=False, help_text="Facultatif.")
    tresorier = PortailUserField(label="Trésorier")
    secretaire = PortailUserField(label="Secrétaire", required=False, help_text="Facultatif.")


class AliasGereForm(forms.ModelForm):
    def __init__(self, club, *args, **kwargs):
        self.ecole = club.ecole
        super(AliasGereForm, self).__init__(*args, **kwargs)

    def clean_prefixe(self):
        prefixe = self.cleaned_data["prefixe"]

        if not re.match(r"^[A-Za-z0-9._-]+$", prefixe):
            raise forms.ValidationError("Le préfixe contient des caractères invalides.")

        if not LdapAlias.objects.filter(cn=prefixe, base=self.ecole.dn):
            raise ValidationError("L’alias {}@{}n’existe pas.".format(prefixe, settings.EMAIL_BDE[self.ecole.id - 1]))

        return prefixe

    class Meta:
        model = AliasGere
        fields = ("prefixe",)


class DomaineForm(forms.Form):
    domaine = forms.CharField(label="Domaine cible", max_length=100)

    def clean_domaine(self):
        domaine = self.cleaned_data["domaine"]

        pattern = re.compile(r"^\w+(\.\w+)+$", re.ASCII)
        if not pattern.match(domaine):
            raise forms.ValidationError("Seuls les caractères alphadécimaux sont autorisés dans les domaines d'alias")

        return domaine

    class Meta:
        fields = ("domaine",)

class LogoForm(forms.Form):
    """Formulaire pour le logo du club"""

    logo = forms.ImageField()

    def clean_logo(self):
        logo = self.cleaned_data["logo"]
        MAX_SIZE = 512  # en Ko

        if logo:
            if logo.content_type != "image/png":
                raise forms.ValidationError("Le format de l'image doit être du PNG !")
            if logo.size > MAX_SIZE * 1024:
                raise forms.ValidationError("La taille de l'image ne doit pas " "dépasser les {} Ko !".format(MAX_SIZE))

        return logo


class ArticleBlogForm(BBCodeModelForm):
    titre = forms.CharField(widget=forms.TextInput(attrs={"size": "64"}))

    class Meta:
        model = ArticleBlog
        fields = ("titre", "contenu")
        bbcode_field = "contenu"
        main_attach_target = "image_accueil"


class ClubDescriptionForm(BBCodeModelForm):
    class Meta:
        model = Club
        fields = ("description", "mini_description")
        bbcode_field = "description"


class ClubForm(forms.Form):
    displayName = forms.CharField(label="Nom", widget=forms.TextInput(attrs={"size": "10"}))
    typeClub = forms.TypedChoiceField(label="Type", choices=TYPES)
    activite = forms.ChoiceField(label="Activité", choices=ACTIVITES)
    local = forms.CharField(widget=forms.TextInput(attrs={"size": "15"}), required=False)
    president = PortailUserField(label="Président")
    blog_act = forms.BooleanField(label="Blog activé", required=False)

    def __init__(self, donnees):
        if "mailAlias" in donnees:
            self.mailAliases = [i for i in donnees["mailAlias"] if i]
        else:
            self.mailAliases = []
            i = 0
            while "mailAlias" + str(i) in donnees:
                mailAlias = donnees["mailAlias" + str(i)].strip()
                if mailAlias:
                    self.mailAliases.append(mailAlias)
                i += 1
        self.mailAliases.append("")

        if "tresorier" in donnees:
            self.tresoriers = [i for i in donnees["tresorier"] if i]
        else:
            self.tresoriers = []
            i = 0
            while "tresorier" + str(i) in donnees:
                tresorier = donnees["tresorier" + str(i)].strip()
                if tresorier:
                    self.tresoriers.append(tresorier)
                i += 1
        self.tresoriers.append("")

        if "vicePresident" in donnees:
            self.vicePresidents = [i for i in donnees["vicePresident"] if i]
        else:
            self.vicePresidents = []
            i = 0
            while "vicePresident" + str(i) in donnees:
                vicePresident = donnees["vicePresident" + str(i)].strip()
                if vicePresident:
                    self.vicePresidents.append(vicePresident)
                i += 1
        self.vicePresidents.append("")

        if "secretaire" in donnees:
            self.secretaires = [i for i in donnees["secretaire"] if i]
        else:
            self.secretaires = []
            i = 0
            while "secretaire" + str(i) in donnees:
                secretaire = donnees["secretaire" + str(i)].strip()
                if secretaire:
                    self.secretaires.append(secretaire)
                i += 1
        self.secretaires.append("")

        forms.Form.__init__(self, donnees.copy())

        elements = list(self.fields.items())[0:4]
        elements_apres = list(self.fields.items())[5:]

        # Insertion d’éléments supplémentaires
        for i, mailAlias in enumerate(self.mailAliases):
            id = "mailAlias" + str(i)
            elements.append(
                (
                    id,
                    forms.EmailField(
                        widget=forms.TextInput(attrs={"size": "30"}),
                        label="Adresse email {}".format(i + 1),
                        required=False,
                    ),
                )
            )
            self.data[id] = mailAlias

        # Le bureau
        elements.append(("president", self.fields["president"]))

        for i, tresorier in enumerate(self.tresoriers):
            id = "tresorier" + str(i)
            elements.append(
                (
                    id,
                    PortailUserField(label="Trésorier {}".format(i + 1), required=i == 0),
                )
            )
            self.data[id] = tresorier

        for i, vicePresident in enumerate(self.vicePresidents):
            id = "vicePresident" + str(i)
            elements.append(
                (
                    id,
                    PortailUserField(label="Vice-Président {}".format(i + 1), required=False),
                )
            )
            self.data[id] = vicePresident

        for i, secretaire in enumerate(self.secretaires):
            id = "secretaire" + str(i)
            elements.append(
                (
                    id,
                    PortailUserField(label="Secrétaire {}".format(i + 1), required=False),
                )
            )
            self.data[id] = secretaire

        self.fields = collections.OrderedDict(elements + elements_apres)

    def is_valid(self):
        valid = super(ClubForm, self).is_valid()

        if not valid:
            return valid

        for i in range(len(self.tresoriers)):
            for j in range(i + 1, len(self.tresoriers)):
                if (
                    self.cleaned_data["tresorier{}".format(i)] == self.cleaned_data["tresorier{}".format(j)]
                    and self.cleaned_data["tresorier{}".format(j)]
                ):
                    valid = False
                    self._errors["tresorier{}".format(j)] = mark_safe(
                        '<ul class="errorlist"><li>'
                        "{} est déjà trésori{} de ce club (redondance)</li></ul>".format(
                            self.cleaned_data["tresorier{}".format(j)].get_full_name(),
                            ("ère" if not self.cleaned_data["tresorier{}".format(j)].get_is_masculin() else "er"),
                        )
                    )

        for i in range(len(self.vicePresidents)):
            for j in range(i + 1, len(self.vicePresidents)):
                if (
                    self.cleaned_data["vicePresident{}".format(i)] == self.cleaned_data["vicePresident{}".format(j)]
                    and self.cleaned_data["vicePresident{}".format(j)]
                ):
                    valid = False
                    self._errors["vicePresident{}".format(j)] = mark_safe(
                        '<ul class="errorlist"><li>'
                        "{} est déjà vice président{} de ce club (redondance)</li></ul>".format(
                            self.cleaned_data["vicePresident{}".format(j)].get_full_name(),
                            ("e" if not self.cleaned_data["vicePresident{}".format(j)].get_is_masculin() else ""),
                        )
                    )

        for i in range(len(self.secretaires)):
            for j in range(i + 1, len(self.secretaires)):
                if (
                    self.cleaned_data["secretaire{}".format(i)] == self.cleaned_data["secretaire{}".format(j)]
                    and self.cleaned_data["secretaire{}".format(j)]
                ):
                    valid = False
                    self._errors["secretaire{}".format(j)] = mark_safe(
                        '<ul class="errorlist"><li>'
                        "{} est déjà secretaire de ce club (redondance)</li></ul>".format(
                            self.cleaned_data["secretaire{}".format(j)].get_full_name()
                        )
                    )

        if not valid:
            return valid

        self.cleaned_data["tresorier"] = []
        for i in range(len(self.tresoriers)):
            if self.cleaned_data["tresorier{}".format(i)]:
                self.cleaned_data["tresorier"].append(self.cleaned_data["tresorier{}".format(i)].username)

        self.cleaned_data["vicePresident"] = []
        for i in range(len(self.vicePresidents)):
            if self.cleaned_data["vicePresident{}".format(i)]:
                self.cleaned_data["vicePresident"].append(self.cleaned_data["vicePresident{}".format(i)].username)

        self.cleaned_data["secretaire"] = []
        for i in range(len(self.secretaires)):
            if self.cleaned_data["secretaire{}".format(i)]:
                self.cleaned_data["secretaire"].append(self.cleaned_data["secretaire{}".format(i)].username)

        self.cleaned_data["mailAlias"] = []
        for i in range(len(self.mailAliases)):
            if self.cleaned_data["mailAlias{}".format(i)]:
                self.cleaned_data["mailAlias"].append(self.cleaned_data["mailAlias{}".format(i)])

        return valid


class EdtClubsForm(forms.ModelForm):
    class Meta:
        model = EdtClubs
        exclude = ("club", "uid")

    def clean(self):
        cleaned_data = self.cleaned_data
        start = cleaned_data.get("start")
        end = cleaned_data.get("end")

        if start and end and start > end:
            self._errors["start"] = ErrorList(["Date / heure de début supérieure à date / heure de fin"])
            self._errors["end"] = ErrorList(["Date / heure de fin inférieure à date / heure de début"])
            del cleaned_data["start"]
            del cleaned_data["end"]

        return cleaned_data
